/*********************************************************************
 *
 * Authors: Paolo Badino   - paolo.badino@cern.ch
 *
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  GLite Data Agents Common: MySqlDAOContext Class
 *
 *  Version info: $Id: MySqlDAOContext.h,v 1.5 2008/11/14 16:04:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 *********************************************************************/
#ifndef GLITE_DATA_AGENTS_DAO_MYSQL_MYSQLDAOCONTEXT_H_
#define GLITE_DATA_AGENTS_DAO_MYSQL_MYSQLDAOCONTEXT_H_

#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/glite-types.h"

#include "mysql.h"
#include <string>
#include <glite/data/config/service/Logger.hxx>

namespace glite {
namespace data  {
namespace agents{
namespace dao   {
namespace mysql {

/**
 * DataAccessObject Context For MySQL
 */
class MySqlDAOContext : public DAOContext {
public:

    /**
     * Constructor
     */
    explicit MySqlDAOContext(const std::string& agentDn = "");

    /**
     * Destructor
     */
    virtual ~MySqlDAOContext();

    // -------------------------------------------------------------------------
    // DAOContext Methods
    // -------------------------------------------------------------------------

    /**
     * Check if the Connection is alive
     */
    virtual void check() /*throw (DAOLogicError, DAOException)*/;

        /**
     * Start a Transaction
     */
    virtual void start() /*throw (DAOLogicError, DAOException)*/;

    /**
     * Commit a Transaction
     */
    virtual void commit() /*throw (DAOLogicError, DAOException)*/;

    /**
     * Rollback a Transaction
     */
    virtual void rollback() /*throw ()*/;

    /**
     * kill context
     */
    virtual void kill() /*throw ()*/;

    // -------------------------------------------------------------------------
    // Utility Methods
    // -------------------------------------------------------------------------
    
    /**
     * Escape a String
     */
    void escape(const std::string& str,std::string& dest);
        
    /**
     * Convert a time value into a Timestamp string representation
     */
    std::string toTimestamp(time_t t);
        
    /**
     * Convert a time value into a Datetime string representation
     */
    std::string toDatetime(time_t t);
        
    /**
     * Convert a Datetime string representation into a time value
     */
    time_t toTimeT(const std::string& datetime);
    
    /**
     * Convert a string representation into a longlong
     */
    longlong toLongLong(const std::string& value);
    
    /**
     * Convert a longlong into a string representation
     */
    std::string toString(longlong value);

    // -------------------------------------------------------------------------
    // Getter Methods
    // -------------------------------------------------------------------------
    
    /**
     * Get MySQL Context
     */
    MYSQL& mySQL() {return m_mySQL;}

    // -------------------------------------------------------------------------
    // Control Methods
    // -------------------------------------------------------------------------

    /**
     * Initialize the mysql context
     */
    void init() /*throw (DAOLogicError, DAOException)*/;

private:
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
    
    /**
     * The MYSQL Connection Object
     */
    MYSQL m_mySQL;
};

} // End namespace mysql
} // End namespace dao
} // End namespace agents
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_AGENTS_DAO_MYSQL_MYSQLDAOCONTEXT_H_
